@@ -6,11 +6,13 @@ module Grape
       attr_accessor :params
       attr_accessor :message_key
 
-      def initialize(args = {})
-        fail 'Params are missing:' unless args.key? :params
-        @params = args[:params]
-        args[:message] = translate_message(args[:message]) if args.key? :message
-        super
+      def initialize(params:, message: nil, **args)
+        @params = params
+        if message
+          @message_key = message if message.is_a?(Symbol)
+          args[:message] = translate_message(message)
+        end
+        super(args)
       end
 
       # remove all the unnecessary stuff from Grape::Exceptions::Base like status
